@@ -2,7 +2,7 @@
    Project: LaternaMagica
    AppController.h
 
-   Copyright (C) 2006-2011 Riccardo Mottola
+   Copyright (C) 2006-2015 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -25,6 +25,7 @@
 
 
 #import <AppKit/AppKit.h>
+
 #import <FileTable.h>
 #import <LMFlipView.h>
 #import "LMWindow.h"
@@ -38,10 +39,12 @@
     IBOutlet LMFlipView   *smallView;
     IBOutlet NSScrollView *scrollView;
     IBOutlet NSButton     *fitButton;
+    IBOutlet NSTextField  *fieldImageCount;
     IBOutlet NSMenuItem   *fullScreenMenuItem;
     IBOutlet NSButton     *fullScreenButton;
     IBOutlet NSMenuItem   *saveAsMenuItem;
     
+    /* save As options */
     IBOutlet NSView        *saveOptionsView;
     IBOutlet NSPopUpButton *fileTypePopUp;
     IBOutlet NSTextField   *jpegCompressionField;
@@ -50,7 +53,7 @@
     BOOL                  scaleToFit;
     NSWindow              *window;
     LMWindow              *fullWindow;
-    NSImageView           *view;
+    LMFlipView            *view;
     LMFlipView            *fullView;
     NSSavePanel           *savePanel;
     
@@ -83,6 +86,10 @@
 - (IBAction)rotateImage180:(id)sender;
 - (IBAction)rotateImage270:(id)sender;
 
+- (void)updateImageCount;
+
+- (NSImage *)rotate: (NSImage *)image byAngle:(unsigned)angle;
+
 - (IBAction)saveImageAs:(id)sender;
 - (IBAction)setCompressionType:(id)sender;
 - (IBAction)setCompressionLevel:(id)sender;
@@ -97,4 +104,6 @@
 - (IBAction)savePreferences:(id)sender;
 - (IBAction)cancelPreferences:(id)sender;
 
+- (void)print:(id)sender;
+
 @end
