@@ -2,7 +2,7 @@
    Project: LaternaMagica
    AppController.m
 
-   Copyright (C) 2006-2012 Riccardo Mottola
+   Copyright (C) 2006-2017 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -23,11 +23,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
-#if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
-#define NSInteger int
-#endif
-
-#if defined(__MINGW__)
+#if defined(__MINGW32__)
 #define srandom srand
 #define random rand
 #endif
@@ -36,8 +32,18 @@
 #include <time.h>
 
 #import "AppController.h"
+#import "LMImage.h"
 #import "PRScale.h"
 
+#if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
+#define NSInteger int
+#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_3
+#define NSImageGamma @"NSImageGamma"
+#define NSImageEXIFData @"NSImageEXIFData"
+#endif
+#endif
+
+
 #define LM_KEY_DESTROYRECYCLE @"DestroyOrRecycle"
 #define LM_KEY_ASKDELETING @"AskBeforeDeleting"
 
@@ -45,17 +51,12 @@
 
 - (id) init
 {
-    if ((self = [super init]))
+  if ((self = [super init]))
     {
-      /* add an observer for the file table view */
-      [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_selectionDidChange:) name:NSTableViewSelectionDidChangeNotification object:fileListView];
-      /* add an observer for the window resize */
-      [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_windowDidResize:) name:NSWindowDidResizeNotification object:window];
-
       /* initialize random number generator */
       srandom(time(NULL));
     }
-    return self;
+  return self;
 }
 
 - (void)awakeFromNib
@@ -76,6 +77,7 @@
 
     [smallView setFrame:[scrollView documentVisibleRect]];
     [smallView setImageAlignment:NSImageAlignTopLeft];
+    [smallWindow setDelegate:smallView];
 
     fullView = [[LMFlipView alloc] initWithFrame:[fullWindow frame]];
     [fullView setImageScaling: NSScaleNone];
@@ -84,9 +86,19 @@
     [fullView setController:self];
     
 
-    // avoid replacing the contentview with a NSControl subclass, thus add a subview instead
+    /* avoid replacing the contentview with a NSControl subclass, thus add a subview instead */
     [[fullWindow contentView] addSubview: fullView];
     [fullWindow setInitialFirstResponder:fullView];
+    [fullWindow setDelegate:fullView];
+
+    /* register the file view as drag destionation */
+    [fileListView registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
+
+    /* add an observer for the file table view */
+    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_selectionDidChange:) name:NSTableViewSelectionDidChangeNotification object:fileListView];
+
+    /* add an observer for the window resize */
+    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_windowDidResize:) name:NSWindowDidResizeNotification object:window];
 }
 
 - (void)dealloc
@@ -98,7 +110,7 @@
 
 - (void)addFile:(NSString *)filename
 {
-    [fileListData addPath:filename];
+  if([fileListData addPathAndRecurse:filename])
     [fileListView reloadData];
 }
 
@@ -116,61 +128,27 @@
     [openPanel setCanChooseDirectories:YES];
     [openPanel setCanChooseFiles:YES];
     if ([openPanel runModalForTypes:NULL] != NSOKButton)
-    {
-        return;
-    }
-
+      return;
+    
     files = [openPanel filenames];
     e = [files objectEnumerator];
     fmgr = [NSFileManager defaultManager];
     while ((filename = (NSString*)[e nextObject]))
-    {
+      {
         attrs = [fmgr fileAttributesAtPath:filename traverseLink:YES];
         if (attrs)
-        {
-            if ([attrs objectForKey:NSFileType] == NSFileTypeDirectory)
-            {
-                NSArray      *dirContents;
-                NSEnumerator *e2;
-                NSString     *filename2;
-                NSDictionary  *attrs2;
-
-                dirContents = [fmgr subpathsAtPath:filename];
-                e2 = [dirContents objectEnumerator];
-                while ((filename2 = (NSString*)[e2 nextObject]))
-                {
-                    NSString *tempName;
-                    NSString *lastPathComponent;
-
-                    lastPathComponent = [filename2 lastPathComponent];
-                    tempName = [filename stringByAppendingPathComponent:filename2];
-                    attrs2 = [[NSFileManager defaultManager] fileAttributesAtPath:tempName traverseLink:YES];
-                    if (attrs2)
-                    {
-                        if ([attrs2 objectForKey:NSFileType] != NSFileTypeDirectory)
-                        {
-                            if (!([lastPathComponent isEqualToString:@".gwdir"] || [lastPathComponent isEqualToString:@".DS_Store"]))
-                            {
-			      /* hide dot files, eventually a preference could be implemented */
-			      if (![lastPathComponent hasPrefix: @"."])
-                                [self addFile:tempName];
-                            }
-                        }
-                    }
-                }
-            } else
-            { /* not a directory */
-                [self addFile:filename];
-            }
-        } else
-        {
+          {
+            [self addFile:filename];
+          }
+        else
+          {
             NSLog(@"open panel did not return a valid path");
-        }
-    }
-  [fileListView selectRow: [fileListView numberOfRows]-1 byExtendingSelection: NO];
+          }
+      }
+    [fileListView selectRow: [fileListView numberOfRows]-1 byExtendingSelection: NO];
 }
 
-// scale image according to options
+// scale image view according to options
 - (void)scaleView:(NSImage *) image
 {
     NSPoint rectOrigin;
@@ -207,21 +185,33 @@
     }
     rectOrigin = NSMakePoint((rectSize.width - [view frame].size.width)/2, (rectSize.height - [view frame].size.height)/2);
     [view setFrameOrigin:rectOrigin];
-    [view setNeedsDisplay:YES];
 }
 
-- (void)changeImage:(NSString *) file
+- (void)changeImage:(LMImage *) image
 {
-    NSImage *image;
+  NSImage *nsImage;
+  NSImage *img;
 
-    image = [[NSImage alloc] initByReferencingFile:file];
-    [self scaleView:image];
-    [view setImage: image];
-    [view setNeedsDisplay:YES];
-    [[view superview] setNeedsDisplay:YES];
-    [window displayIfNeeded];
-    [image release];
-    [window setTitleWithRepresentedFilename:file];
+  nsImage = [[NSImage alloc] initByReferencingFile:[image path]];
+  
+  [nsImage autorelease];
+  img = nsImage;
+
+  if ([image rotation] > 0)
+    {
+      NSImage *destImage;
+
+      destImage = [self rotate: nsImage byAngle:[image rotation]];
+      img = destImage;
+    }
+  [self scaleView: img];
+  [view setImage: img];
+
+  [view setNeedsDisplay:YES];
+  [[view superview] setNeedsDisplay:YES];
+  
+  /* we don't need to update the full-screen window title */
+  [smallWindow setTitleWithRepresentedFilename:[image name]];
 }
 
 - (IBAction)setScaleToFit:(id)sender
@@ -238,6 +228,7 @@
     [scrollView setHasVerticalScroller:!scaleToFit];
     [scrollView setHasHorizontalScroller:!scaleToFit];
     [self scaleView:[view image]];
+    [view setNeedsDisplay:YES];
 }
 
 /** method called as a notification from the selection change */
@@ -249,7 +240,7 @@
     table = [notif object];
     selectedRow = [table selectedRow];
     if (selectedRow >= 0)
-        [self changeImage:[fileListData pathAtIndex:selectedRow]];
+        [self changeImage:[fileListData imageAtIndex:selectedRow]];
 }
 
 /** method called as a notification from the window resize
@@ -264,12 +255,16 @@
 {
     NSImage *image;
 
-    image = [view image];
+    /* trick for GS so that we don't have order problems with GWorkspace */
+    [window makeKeyAndOrderFront: self];
     
     /* we choose not to respond to key events if not in fullscreen */
     if ([sender isKindOfClass:[NSEvent class]] && [fullScreenMenuItem state] == NSOffState)
         return;
 
+    image = [view image];
+    [image retain];
+
     /* check the sender and set the other item accordingly */    
     if (sender == fullScreenButton)
         [fullScreenMenuItem setState:[fullScreenButton state]];
@@ -284,21 +279,22 @@
 
     if ([fullScreenButton state] == NSOnState)
     {
+      [smallView setImage: nil];
         [fullWindow setLevel: NSScreenSaverWindowLevel];
         window = fullWindow;
         view = fullView;
     } else
     {
+      [fullView setImage: nil];
         [fullWindow orderOut:self];
         window = smallWindow;
         view = smallView;
     }
-    [self setScaleToFit: self];
-    [self scaleView: image];
     [view setImage: image];
+    [image release];
+    [self setScaleToFit: self];
     [view setNeedsDisplay:YES];
     [[view superview] setNeedsDisplay:YES];
-    [window displayIfNeeded];
     [window makeKeyAndOrderFront: self];
 }
 
@@ -343,11 +339,11 @@
 
 - (IBAction)removeAllImages:(id)sender
 {
-  int rows;
-  int i;
+  NSInteger rows;
+  NSInteger i;
   
   rows = [fileListView numberOfRows];
-  for (i = ([fileListView numberOfRows]  - 1); i >= 0 ; i--)
+  for (i = (rows  - 1); i >= 0 ; i--)
     [fileListData removeObjectAtIndex: i];
   [fileListView reloadData];
   [view setImage: nil];
@@ -356,12 +352,15 @@
 
 - (IBAction)scrambleList:(id)sender
 {
+  NSInteger selRow;
   [fileListData scrambleObjects];
   [fileListView reloadData];
-  
+
   /* we reload the image directly without calling selectRow,
      the selected row did not actually change and no notification triggers */
-  [self changeImage: [fileListData pathAtIndex: [fileListView selectedRow]]];
+  selRow = [fileListView selectedRow];
+  if (selRow > 0)
+    [self changeImage: [fileListData imageAtIndex: (NSUInteger)selRow]];
 }
 
 - (IBAction)removeImage:(id)sender
@@ -387,7 +386,7 @@
                 if (sr >= rows)
                     [fileListView selectRow: rows-1 byExtendingSelection: NO];
                 else
-                    [self changeImage: [fileListData pathAtIndex: sr]];
+                    [self changeImage: [fileListData imageAtIndex: sr]];
             
             } else
             {
@@ -424,7 +423,7 @@
 	{
 	  NSWorkspace *ws;
 	  NSString *fileOperation;
-	  int opTag;
+	  NSInteger opTag;
 	  NSString *folder;
 	  NSString *file;
 
@@ -451,180 +450,179 @@
 
 - (IBAction)rotateImage90:(id)sender
 {
-    NSImage *srcImage;
-    NSBitmapImageRep *srcImageRep;
-    NSImage *destImage;
-    NSBitmapImageRep *destImageRep;
-    int x, y;
-    int w, h;
-    int s;
-    int srcSamplesPerPixel;
-    int destSamplesPerPixel;
-    unsigned char *srcData;
-    unsigned char *destData;
-    unsigned char *p1, *p2;
+  NSImage *destImage;
 
-    srcImage = [view image];
-    /* get source image representation and associated information */
-    srcImageRep = [NSBitmapImageRep imageRepWithData:[srcImage TIFFRepresentation]];
+  LMImage *imageInfo;
 
-    w = [srcImageRep pixelsWide];
-    h = [srcImageRep pixelsHigh];
-    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
-    destSamplesPerPixel = srcSamplesPerPixel;
-    destImage = [[NSImage alloc] initWithSize:NSMakeSize(h, w)]; /* we swap h and w */
-    destImageRep = [[NSBitmapImageRep alloc]
-                initWithBitmapDataPlanes:NULL
-                              pixelsWide:h
-                              pixelsHigh:w
-                           bitsPerSample:[srcImageRep bitsPerSample]
-                         samplesPerPixel:[srcImageRep samplesPerPixel]
-                                hasAlpha:[srcImageRep hasAlpha]
-                                isPlanar:[srcImageRep isPlanar]
-                          colorSpaceName:[srcImageRep colorSpaceName]
-                             bytesPerRow:h*destSamplesPerPixel  // we need to set this because otherwise mac > 10.4 will set a padded value
-                            bitsPerPixel:0];
+  imageInfo = [fileListData imageAtIndex:[fileListView selectedRow]];
+  [imageInfo setRotation: 90];
 
-    srcData = [srcImageRep bitmapData];
-    destData = [destImageRep bitmapData];
-    
-    for (y = 0; y < h; y++)
-        for (x = 0; x < w; x++)
-        {
-            p1 = srcData + srcSamplesPerPixel * (y * w + x);
-            p2 = destData + destSamplesPerPixel * ((w-x-1) * h + y);
-            for (s = 0; s < srcSamplesPerPixel; s++)
-                p2[s] = p1[s];
-        }
+  destImage = [self rotate: [view image] byAngle:90];
 
-    [destImage addRepresentation:destImageRep];
-    [destImageRep release];
 
-    [self scaleView:destImage];
-    [view setImage: destImage];
-    [view setNeedsDisplay:YES];
-    [[view superview] setNeedsDisplay:YES];
-    [window displayIfNeeded];
-    [destImage release];
+  [self scaleView:destImage];
+  [view setImage: destImage];
+  [view setNeedsDisplay:YES];
+  [[view superview] setNeedsDisplay:YES];
 }
 
 - (IBAction)rotateImage180:(id)sender
 {
-    NSImage *srcImage;
-    NSBitmapImageRep *srcImageRep;
-    NSImage *destImage;
-    NSBitmapImageRep *destImageRep;
-    int x, y;
-    int w, h;
-    int s;
-    int srcSamplesPerPixel;
-    int destSamplesPerPixel;
-    unsigned char *srcData;
-    unsigned char *destData;
-    unsigned char *p1, *p2;
+  NSImage *destImage;
 
-    srcImage = [view image];
-    /* get source image representation and associated information */
-    srcImageRep = [NSBitmapImageRep imageRepWithData:[srcImage TIFFRepresentation]];
+  LMImage *imageInfo;
 
-    w = [srcImageRep pixelsWide];
-    h = [srcImageRep pixelsHigh];
-    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
-    destSamplesPerPixel = srcSamplesPerPixel;
+  imageInfo = [fileListData imageAtIndex:[fileListView selectedRow]];
+  [imageInfo setRotation: 180];
 
-    destImage = [[NSImage alloc] initWithSize:NSMakeSize(w, h)];
-    destImageRep = [[NSBitmapImageRep alloc]
-                initWithBitmapDataPlanes:NULL
-                              pixelsWide:w
-                              pixelsHigh:h
-                           bitsPerSample:[srcImageRep bitsPerSample]
-                         samplesPerPixel:[srcImageRep samplesPerPixel]
-                                hasAlpha:[srcImageRep hasAlpha]
-                                isPlanar:[srcImageRep isPlanar]
-                          colorSpaceName:[srcImageRep colorSpaceName]
-                             bytesPerRow:w*destSamplesPerPixel
-                            bitsPerPixel:0];
+  destImage = [self rotate: [view image] byAngle:180];
 
-    srcData = [srcImageRep bitmapData];
-    destData = [destImageRep bitmapData];
-
-    for (y = 0; y < h; y++)
-        for (x = 0; x < w; x++)
-        {
-            p1 = srcData + srcSamplesPerPixel * (y * w + x);
-            p2 = destData + srcSamplesPerPixel * ((h-y-1) * w + (w-x-1));
-            for (s = 0; s < srcSamplesPerPixel; s++)
-                p2[s] = p1[s];
-        }
-
-    [destImage addRepresentation:destImageRep];
-    [destImageRep release];
-
-    [self scaleView:destImage];
-    [view setImage: destImage];
-    [window displayIfNeeded];
-    [destImage release];
+  [self scaleView:destImage];
+  [view setImage: destImage];
+  [view setNeedsDisplay:YES];
+  [[view superview] setNeedsDisplay:YES];
 }
 
 
 - (IBAction)rotateImage270:(id)sender
 {
-    NSImage *srcImage;
-    NSBitmapImageRep *srcImageRep;
-    NSImage *destImage;
-    NSBitmapImageRep *destImageRep;
-    int x, y;
-    int w, h;
-    int s;
-    int srcSamplesPerPixel;
-    int destSamplesPerPixel;
-    unsigned char *srcData;
-    unsigned char *destData;
-    unsigned char *p1, *p2;
+  NSImage *destImage;
+  
+  LMImage *imageInfo;
 
-    srcImage = [view image];
-    /* get source image representation and associated information */
-    srcImageRep = [NSBitmapImageRep imageRepWithData:[srcImage TIFFRepresentation]];
+  imageInfo = [fileListData imageAtIndex:[fileListView selectedRow]];
+  [imageInfo setRotation: 270];
 
-    w = [srcImageRep pixelsWide];
-    h = [srcImageRep pixelsHigh];
-    srcSamplesPerPixel = [srcImageRep samplesPerPixel];
-    destSamplesPerPixel = srcSamplesPerPixel;
+  destImage = [self rotate: [view image] byAngle: 270];
 
-    destImage = [[NSImage alloc] initWithSize:NSMakeSize(h, w)]; /* we swap h and w */
-    destImageRep = [[NSBitmapImageRep alloc]
-                initWithBitmapDataPlanes:NULL
-                              pixelsWide:h
-                              pixelsHigh:w
-                           bitsPerSample:[srcImageRep bitsPerSample]
-                         samplesPerPixel:[srcImageRep samplesPerPixel]
-                                hasAlpha:[srcImageRep hasAlpha]
-                                isPlanar:[srcImageRep isPlanar]
-                          colorSpaceName:[srcImageRep colorSpaceName]
-                             bytesPerRow:h*destSamplesPerPixel
-                            bitsPerPixel:0];
+  [self scaleView:destImage];
+  [view setImage: destImage];
+  [[view superview] setNeedsDisplay:YES];
+}
 
-    srcData = [srcImageRep bitmapData];
-    destData = [destImageRep bitmapData];
+- (NSImage *)rotate: (NSImage *)image byAngle:(unsigned)angle
+{
+  NSBitmapImageRep *srcImageRep;
+  NSImage *destImage;
+  NSBitmapImageRep *destImageRep;
+  NSMutableDictionary *imgProps;
+  NSInteger x, y;
+  NSInteger w, h;
+  NSInteger newW, newH;
+  NSInteger srcSamplesPerPixel;
+  NSInteger destSamplesPerPixel;
+  unsigned char *srcData;
+  unsigned char *destData;
+  unsigned char *p1, *p2;
+  NSSize oldRepSize, newRepSize;
+  NSSize oldImgSize, newImgSize;
+  register NSInteger srcBytesPerPixel;
+  register NSInteger destBytesPerPixel;
+  register NSInteger srcBytesPerRow;
+  register NSInteger destBytesPerRow;
+  
+  /* get source image representation and associated information */
+  oldImgSize = [image size];
+  srcImageRep = [NSBitmapImageRep imageRepWithData:[image TIFFRepresentation]];
+  oldRepSize = [srcImageRep size];
+  w = [srcImageRep pixelsWide];
+  h = [srcImageRep pixelsHigh];
+  if (angle == 90 || angle == 270)
+    {
+      newH = w;
+      newW = h;
+      newImgSize = NSMakeSize(oldImgSize.height, oldImgSize.width);
+      newRepSize = NSMakeSize(oldRepSize.height, oldRepSize.width);
+    }
+  else
+    {
+      newH = h;
+      newW = w;
+      newImgSize = oldImgSize;
+      newRepSize = oldRepSize;
+    }
 
-    for (y = 0; y < h; y++)
+  srcSamplesPerPixel = [srcImageRep samplesPerPixel];
+  destSamplesPerPixel = srcSamplesPerPixel;
+  srcBytesPerRow = [srcImageRep bytesPerRow];
+  srcBytesPerPixel = [srcImageRep bitsPerPixel] / 8;  srcSamplesPerPixel = [srcImageRep samplesPerPixel];
+
+  imgProps = [[NSMutableDictionary alloc] init];
+  [imgProps setValue:[srcImageRep valueForProperty:NSImageCompressionMethod] forKey:NSImageCompressionMethod];
+  [imgProps setValue:[srcImageRep valueForProperty:NSImageCompressionFactor] forKey:NSImageCompressionFactor];
+  [imgProps setValue:[srcImageRep valueForProperty:NSImageEXIFData] forKey:NSImageEXIFData];
+  [imgProps setValue:[srcImageRep valueForProperty:NSImageGamma] forKey:NSImageGamma];
+  [imgProps setValue:[srcImageRep valueForProperty:NSImageColorSyncProfileData] forKey:NSImageColorSyncProfileData];
+  NSLog(@"Properties: %@", imgProps);
+  
+  destSamplesPerPixel = srcSamplesPerPixel;
+  destImage = [[NSImage alloc] initWithSize:newImgSize];
+  destImageRep = [[NSBitmapImageRep alloc]
+		   initWithBitmapDataPlanes:NULL
+				 pixelsWide:newW
+				 pixelsHigh:newH
+			      bitsPerSample:[srcImageRep bitsPerSample]
+			    samplesPerPixel:[srcImageRep samplesPerPixel]
+				   hasAlpha:[srcImageRep hasAlpha]
+				   isPlanar:[srcImageRep isPlanar]
+			     colorSpaceName:[srcImageRep colorSpaceName]
+				bytesPerRow:0
+			       bitsPerPixel:0];
+  [destImageRep setSize:newRepSize];
+  [destImageRep setProperty:NSImageEXIFData withValue:[imgProps valueForKey:NSImageEXIFData]];
+  [destImageRep setProperty:NSImageColorSyncProfileData withValue:[imgProps valueForKey:NSImageColorSyncProfileData]];
+  [destImageRep setProperty:NSImageGamma withValue:[imgProps valueForKey:NSImageGamma]];
+  [imgProps release];
+
+  srcData = [srcImageRep bitmapData];
+  destData = [destImageRep bitmapData];
+  destBytesPerRow = [destImageRep bytesPerRow];
+  destBytesPerPixel = [destImageRep bitsPerPixel] / 8;
+    
+  if (angle == 90)
+    {
+      for (y = 0; y < h; y++)
         for (x = 0; x < w; x++)
-        {
-            p1 = srcData + srcSamplesPerPixel * (y * w + x);
-            p2 = destData + destSamplesPerPixel * (x * h + (h-y-1));
+	  {
+            unsigned s;
+            
+            p1 = srcData + srcBytesPerRow * y  + srcBytesPerPixel * x;
+            p2 = destData + destBytesPerRow * (w-x-1) + destBytesPerPixel * y;
             for (s = 0; s < srcSamplesPerPixel; s++)
-                p2[s] = p1[s];
-        }
+	      p2[s] = p1[s];
+	  }
+    }
+  else if (angle == 180)
+    {
+      for (y = 0; y < h; y++)
+        for (x = 0; x < w; x++)
+	  {
+            unsigned s;
+            
+            p1 = srcData + srcBytesPerRow * y  + srcBytesPerPixel * x;
+            p2 = destData + destBytesPerRow * (h-y-1) + destBytesPerPixel * (w-x-1);
+            for (s = 0; s < srcSamplesPerPixel; s++)
+	      p2[s] = p1[s];
+	  }
+    }
+  else if (angle == 270)
+    {
+      for (y = 0; y < h; y++)
+        for (x = 0; x < w; x++)
+	  {
+            unsigned s;
+            
+            p1 = srcData + srcBytesPerRow * y  + srcBytesPerPixel * x;
+            p2 = destData + destBytesPerRow * x + destBytesPerPixel * (h-y-1);
+            for (s = 0; s < srcSamplesPerPixel; s++)
+	      p2[s] = p1[s];
+	  }
+    }
+  [destImage addRepresentation:destImageRep];
+  [destImageRep release];
 
-    [destImage addRepresentation:destImageRep];
-    [destImageRep release];
-
-    [self scaleView:destImage];
-    [view setImage: destImage];
-    [view setNeedsDisplay:YES];
-    [[view superview] setNeedsDisplay:YES];
-    [window displayIfNeeded];
-    [destImage release];
+  [destImage autorelease];
+  return destImage;
 }
 
 - (BOOL)validateMenuItem:(id)sender
@@ -638,6 +636,11 @@
     return YES;
 }
 
+- (void)updateImageCount
+{
+  [fieldImageCount setIntValue:(int)[fileListData imageCount]];
+}
+
 - (IBAction)saveImageAs:(id)sender
 {
     NSImage *srcImage;
@@ -773,7 +776,7 @@
   NSString *destFileName;
   NSString *destFileExtension;
   NSString *destFolder;
-  int i;
+  NSInteger i;
   
   givenHeight = [fieldHeight intValue];
   givenWidth = [fieldWidth intValue];
@@ -789,24 +792,128 @@
   [exporterPanel displayIfNeeded];
   for (i = 0; i < [fileListView numberOfRows]; i++)
     {
-      NSString *fullOrigPath;
-      int newW, newH;
+      NSInteger newW, newH;
       double aspectRatio;
       NSBitmapImageRep *scaledImageRep;
       NSAutoreleasePool *pool;
+      LMImage *lmImage;
+
+      lmImage = [fileListData imageAtIndex:i];
 
       /* create a local pool to avoid the autorelease to grow too much */
       pool = [[NSAutoreleasePool alloc] init];
-      fullOrigPath = [fileListData pathAtIndex:i];
-      origFileName = [fullOrigPath lastPathComponent];
-      filenameNoExtension = [origFileName stringByDeletingPathExtension];
-      
+      origFileName = [lmImage name];
+      filenameNoExtension = [origFileName stringByDeletingPathExtension];   
 
-      srcImage = [[NSImage alloc] initByReferencingFile:fullOrigPath];
-      srcImageRep = [NSBitmapImageRep imageRepWithData:[srcImage TIFFRepresentation]];
+      srcImage = [[NSImage alloc] initByReferencingFile:[lmImage path]];
+      if ([lmImage rotation] > 0)
+        {
+          NSImage *rotImage;
 
-      newW = [srcImageRep size].width;
-      newH = [srcImageRep size].height;
+          rotImage = [self rotate:srcImage byAngle:[lmImage rotation]];
+          [rotImage retain];
+          [srcImage release];
+          srcImage = rotImage;
+        }
+      srcImageRep = [[srcImage representations] objectAtIndex:0];;
+
+#if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_3)
+      /* since 10.4 we have different Alpha format position, let's convert it */
+      NSMutableDictionary *imgProps;
+
+      if ([srcImageRep bitmapFormat] != 0)
+        {
+          NSInteger x, y;
+          NSInteger w, h;
+          BOOL alphaFirst;
+          NSImage *destImage;
+          NSBitmapImageRep *destImageRep;
+          NSInteger           srcBytesPerRow;
+          NSInteger           destBytesPerRow;
+          NSInteger           srcBytesPerPixel;
+          NSInteger           destBytesPerPixel;
+          
+          NSLog(@"We have a non-standard format, let's try to convert it");
+          alphaFirst = [srcImageRep bitmapFormat] & NSAlphaFirstBitmapFormat;
+
+          if ([srcImageRep bitsPerSample] == 8)
+            {
+              unsigned char    *srcData;
+              unsigned char    *destData;
+              unsigned char    *p1;
+              unsigned char    *p2;
+
+              /* swap Alpha is hopefully only for chunky images */
+              if (alphaFirst)
+                {
+                  imgProps = [[NSMutableDictionary alloc] init];
+                  [imgProps setValue:[srcImageRep valueForProperty:NSImageCompressionMethod] forKey:NSImageCompressionMethod];
+                  [imgProps setValue:[srcImageRep valueForProperty:NSImageCompressionFactor] forKey:NSImageCompressionFactor];
+                  [imgProps setValue:[srcImageRep valueForProperty:NSImageEXIFData] forKey:NSImageEXIFData];
+                  
+                  w = [srcImageRep pixelsWide];
+                  h = [srcImageRep pixelsHigh];
+
+                  srcBytesPerRow = [srcImageRep bytesPerRow];
+                  srcBytesPerPixel = [srcImageRep bitsPerPixel] / 8;
+                  destImage = [[NSImage alloc] initWithSize:NSMakeSize(w, h)];
+                  destImageRep = [[NSBitmapImageRep alloc]
+                                  initWithBitmapDataPlanes:NULL
+                                                pixelsWide:w
+                                                pixelsHigh:h
+                                             bitsPerSample:8
+                                           samplesPerPixel:[srcImageRep samplesPerPixel]
+                                                  hasAlpha:[srcImageRep hasAlpha]
+                                                  isPlanar:NO
+                                            colorSpaceName:[srcImageRep colorSpaceName]
+                                               bytesPerRow:0
+                                              bitsPerPixel:0];
+                  
+                  destBytesPerRow = [destImageRep bytesPerRow];
+                  destBytesPerPixel = [destImageRep bitsPerPixel] / 8;
+                  srcData = [srcImageRep bitmapData];
+                  destData = [destImageRep bitmapData];
+                  if ([srcImageRep samplesPerPixel] == 2)
+                    {
+                      for (y = 0; y < h; y++)
+                        for (x = 0; x < w; x++)
+                          {
+                            p1 = srcData + srcBytesPerRow * y + srcBytesPerPixel * x;
+                            p2 = destData + destBytesPerRow * y + destBytesPerPixel * x;
+                            p2[0] = p1[1];
+                            p2[1] = p1[0];
+                          }
+                    }
+                  else
+                    {
+                      for (y = 0; y < h; y++)
+                        for (x = 0; x < w; x++)
+                          {
+                            p1 = srcData + srcBytesPerRow * y + srcBytesPerPixel * x;
+                            p2 = destData + destBytesPerRow * y + destBytesPerPixel * x;
+                            p2[0] = p1[1];
+                            p2[1] = p1[2];
+                            p2[2] = p1[3];
+                            p2[3] = p1[0];
+                          }
+                    }
+                  [destImageRep setProperty:NSImageEXIFData withValue:[imgProps objectForKey:NSImageEXIFData]];
+		  [destImageRep setSize:[srcImageRep size]];
+                  [destImage addRepresentation:destImageRep];
+                  [destImageRep release];
+                  [srcImage release];
+                  srcImage = destImage;
+                  [imgProps release];
+                }
+            }
+          else /* for 16 bit */
+            {
+            }
+        }
+#endif
+
+      newW = [srcImageRep pixelsWide];
+      newH = [srcImageRep pixelsHigh];
       aspectRatio = (double)newW / newH;
       NSLog(@"aspect ratio: %f", aspectRatio);
       switch([popupConstraints indexOfSelectedItem])
@@ -831,7 +938,7 @@
             }
 	  break;
 	case 4: /* largest side */
-	  if ([srcImageRep size].height > [srcImageRep size].width)
+	  if ([srcImageRep pixelsHigh] > [srcImageRep pixelsWide])
 	    {
 	      newH = givenWidth;
 	      newW = givenWidth * aspectRatio;
@@ -845,8 +952,8 @@
 	default:
 	  NSLog(@"Unexpected constraint selection value.");
 	}
-      NSLog(@"%d %d", newW, newH);
-      if (newW == [srcImageRep size].width)
+
+      if (newW == [srcImageRep pixelsWide])
 	{
 	  NSLog(@"nothing");
 	  scaledImageRep = srcImageRep;
@@ -932,4 +1039,10 @@
   [prefPanel performClose:self];
 }
 
+/* printing */
+- (void)print:(id)sender
+{
+  [view print:sender];
+}
+
 @end
