@@ -2,7 +2,7 @@
    Project: LaternaMagica
    FileTable.h
 
-   Copyright (C) 2006-2011 Riccardo Mottola
+   Copyright (C) 2006-2013 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -27,18 +27,32 @@
 
 #import <AppKit/AppKit.h>
 
+@class LMImage;
+@class AppController;
+
+#if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
+#define NSUInteger unsigned
+#define NSInteger int
+#endif
 
 @interface FileTable : NSObject
 {
-    NSMutableArray *fileNames;
-    NSMutableArray *filePaths;
+  /* files that will not be added */
+  NSArray *filesToIgnore;
+
+  NSMutableArray *images;
+
+  IBOutlet AppController *appController;
 }
 
+- (BOOL)addPathAndRecurse: (NSString*)path;
 - (void)addPath:(NSString *)filename;
-- (NSString *)pathAtIndex :(int)index;
+- (LMImage *)imageAtIndex :(NSUInteger)index;
+- (NSString *)pathAtIndex :(NSUInteger)index;
+- (NSUInteger)imageCount;
 
 /** removes an element at given index */
-- (void)removeObjectAtIndex:(int)index;
+- (void)removeObjectAtIndex:(NSUInteger)index;
 
 /** shuffles the elements randomly */
 - (void)scrambleObjects;
