@@ -26,9 +26,13 @@
 
 #import <AppKit/AppKit.h>
 
+#if !defined (GNUSTEP) &&  (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
+@protocol NSWindowDelegate;
+#endif
+
 @class AppController;
 
-@interface LMFlipView : NSImageView
+@interface LMFlipView : NSImageView <NSWindowDelegate>
 {
     IBOutlet AppController *controller;
 }
