@@ -10,18 +10,26 @@
 
 #import <AppKit/AppKit.h>
 
+@interface PRCProgress : NSObject
+{
+  IBOutlet NSWindow            *progressPanel;
+  IBOutlet NSProgressIndicator *progressBar;
+  IBOutlet NSTextField         *activityDescription;
+}
+
+- (IBAction)showProgress:(id)sender;
+
+@end
+
+
 #define NEAREST_NEIGHBOUR 1
 #define BILINEAR 2
 
-@class PRCProgress;
 
 @interface PRScale : NSObject
 {
-  int progressSteps;
-  int totalProgressSteps;
-  id progPanel;
 }
 
-- (NSImage *)scaleImage :(NSImage *)srcImage :(int)sizeX :(int)sizeY :(int)method :(id)prPanel;
+- (NSImage *)scaleImage :(NSImage *)srcImage :(int)sizeX :(int)sizeY :(int)method :(PRCProgress *)prPanel;
 
 @end
